﻿using System;
using System.Collections.Generic;

namespace HeuristicLab.Algorithms.PushGP
{
    public static class SymbolTable
    {
        private static Dictionary<string, OpCode> symbolTable = new Dictionary<string, OpCode>
        {
            #region Code
            { Symbols.CodeIf, OpCode.CodeIf },
            { Symbols.CodeDefine, OpCode.CodeDefine },
            { Symbols.CodeDuplicate, OpCode.CodeDuplicate },
            { Symbols.CodeQuote, OpCode.CodeQuote },
            { Symbols.CodeDo, OpCode.CodeDo },
            { Symbols.CodeEquals, OpCode.CodeEquals },
            { Symbols.CodeAppend, OpCode.CodeAppend },
            { Symbols.CodeAtom, OpCode.CodeAtom },
            { Symbols.CodeCar, OpCode.CodeCar },
            { Symbols.CodeCdr, OpCode.CodeCdr },
            { Symbols.CodeCons, OpCode.CodeCons },
            { Symbols.CodeContainer, OpCode.CodeContainer },
            { Symbols.CodeContains, OpCode.CodeContains },
            { Symbols.CodeDefinition, OpCode.CodeDefinition },
            { Symbols.CodeDiscrepancy, OpCode.CodeDiscrepancy },
            { Symbols.CodeDoX, OpCode.CodeDoX },
            { Symbols.CodeDoXCount, OpCode.CodeDoXCount },
            { Symbols.CodeDoXRange, OpCode.CodeDoXRange },
            { Symbols.CodeDoXTimes, OpCode.CodeDoXTimes },
            { Symbols.CodeExtract, OpCode.CodeExtract },
            { Symbols.CodeFlush, OpCode.CodeFlush },
            { Symbols.CodeFromFloat, OpCode.CodeFromFloat },
            { Symbols.CodeFromInteger, OpCode.CodeFromInteger },
            { Symbols.CodeFromName, OpCode.CodeFromName },
            { Symbols.CodeInsert, OpCode.CodeInsert },
            { Symbols.CodeInstructions, OpCode.CodeInstructions },
            { Symbols.CodeLength, OpCode.CodeLength },
            { Symbols.CodeList, OpCode.CodeList },
            { Symbols.CodeMember, OpCode.CodeMember },
            { Symbols.CodeNoop, OpCode.CodeNoop },
            { Symbols.CodeNth, OpCode.CodeNth },
            { Symbols.CodeNthCdr, OpCode.CodeNthCdr },
            { Symbols.CodeNull, OpCode.CodeNull },
            { Symbols.CodePosition, OpCode.CodePosition },
            { Symbols.CodeRand, OpCode.CodeRand },
            { Symbols.CodeRot, OpCode.CodeRot },
            { Symbols.CodeShove, OpCode.CodeShove },
            { Symbols.CodeSize, OpCode.CodeSize },
            { Symbols.CodeStackDepth, OpCode.CodeStackDepth },
            { Symbols.CodeSubst, OpCode.CodeSubst },
            { Symbols.CodeSwap, OpCode.CodeSwap },
            { Symbols.CodeYank, OpCode.CodeYank },
            { Symbols.CodeYankDuplicate, OpCode.CodeYankDuplicate },
            #endregion Code

            #region Exec
            { Symbols.ExecIf, OpCode.ExecIf },
            { Symbols.ExecDefine, OpCode.ExecDefine },
            { Symbols.ExecDuplicate, OpCode.ExecDuplicate },
            { Symbols.ExecPop, OpCode.ExecPop },
            { Symbols.ExecY, OpCode.ExecY },
            { Symbols.ExecDoXRange, OpCode.ExecDoXRange },
            { Symbols.ExecEquals, OpCode.ExecEquals },
            { Symbols.ExecDoXCount, OpCode.ExecDoXCount },
            { Symbols.ExecDoXTimes, OpCode.ExecDoXTimes },
            { Symbols.ExecFlush, OpCode.ExecFlush },
            { Symbols.ExecK, OpCode.ExecK },
            { Symbols.ExecRot, OpCode.ExecRot },
            { Symbols.ExecS, OpCode.ExecS },
            { Symbols.ExecShove, OpCode.ExecShove },
            { Symbols.ExecStackDepth, OpCode.ExecStackDepth },
            { Symbols.ExecSwap, OpCode.ExecSwap },
            { Symbols.ExecYank, OpCode.ExecYank },
            { Symbols.ExecYankDuplicate, OpCode.ExecYankDuplicate },
            #endregion Exec

            #region Name
            { Symbols.NamePop, OpCode.NamePop },
            { Symbols.NameDuplicate, OpCode.NameDuplicate },
            { Symbols.NameEquals, OpCode.NameEquals },
            { Symbols.NameFlush, OpCode.NameFlush },
            { Symbols.NameQuote, OpCode.NameQuote },
            { Symbols.NameRand, OpCode.NameRand },
            { Symbols.NameRandBoundName, OpCode.NameRand },
            { Symbols.NameRot, OpCode.NameRot },
            { Symbols.NameShove, OpCode.NameShove },
            { Symbols.NameStackDepth, OpCode.NameStackDepth },
            { Symbols.NameSwap, OpCode.NameSwap },
            { Symbols.NameYank, OpCode.NameYank },
            { Symbols.NameYankDuplicate, OpCode.NameYankDuplicate },
            #endregion Name

            #region Boolean
            { Symbols.BooleanPop, OpCode.BooleanPop },
            { Symbols.BooleanDuplicate, OpCode.BooleanDuplicate },
            { Symbols.BooleanAnd, OpCode.BooleanAnd },
            { Symbols.BooleanOr, OpCode.BooleanOr },
            { Symbols.BooleanEquals, OpCode.BooleanEquals },
            { Symbols.BooleanFlush, OpCode.BooleanFlush },
            { Symbols.BooleanFromFloat, OpCode.BooleanFromFloat },
            { Symbols.BooleanFromInteger, OpCode.BooleanFromInteger },
            { Symbols.BooleanNot, OpCode.BooleanNot },
            { Symbols.BooleanRand, OpCode.BooleanRand },
            { Symbols.BooleanRot, OpCode.BooleanRot },
            { Symbols.BooleanShove, OpCode.BooleanShove },
            { Symbols.BooleanStackDepth, OpCode.BooleanStackDepth },
            { Symbols.BooleanSwap, OpCode.BooleanSwap },
            { Symbols.BooleanYank, OpCode.BooleanYank },
            { Symbols.BooleanYankDuplicate, OpCode.BooleanYankDuplicate },
            #endregion Boolean

            #region Integer
            { Symbols.IntegerPop, OpCode.IntegerPop },
            { Symbols.IntegerDuplicate, OpCode.IntegerDuplicate },
            { Symbols.IntegerDefine, OpCode.IntegerDefine },
            { Symbols.IntegerAdd, OpCode.IntegerAdd },
            { Symbols.IntegerSubtract, OpCode.IntegerSubtract },
            { Symbols.IntegerMultiply, OpCode.IntegerMultiply },
            { Symbols.IntegerDivide, OpCode.IntegerDivide },
            { Symbols.IntegerSmallerThan, OpCode.IntegerSmallerThan },
            { Symbols.IntegerGreaterThan, OpCode.IntegerGreaterThan },
            { Symbols.IntegerEquals, OpCode.IntegerEquals },
            { Symbols.IntegerMin, OpCode.IntegerMin },
            { Symbols.IntegerMax, OpCode.IntegerMax },
            { Symbols.IntegerModulo, OpCode.IntegerModulo },
            { Symbols.IntegerFlush, OpCode.IntegerFlush },
            { Symbols.IntegerFromBoolean, OpCode.IntegerFromBoolean },
            { Symbols.IntegerFromFloat, OpCode.IntegerFromFloat },
            { Symbols.IntegerRand, OpCode.IntegerRand },
            { Symbols.IntegerRot, OpCode.IntegerRot },
            { Symbols.IntegerShove, OpCode.IntegerShove },
            { Symbols.IntegerStackDepth, OpCode.IntegerStackDepth },
            { Symbols.IntegerSwap, OpCode.IntegerSwap },
            { Symbols.IntegerYank, OpCode.IntegerYank },
            { Symbols.IntegerYankDuplicate, OpCode.IntegerYankDuplicate },
            #endregion Integer

            #region Float
            { Symbols.FloatPop, OpCode.FloatPop },
            { Symbols.FloatDuplicate, OpCode.FloatDuplicate },
            { Symbols.FloatDefine, OpCode.FloatDefine },
            { Symbols.FloatAdd, OpCode.FloatAdd },
            { Symbols.FloatSubtract, OpCode.FloatSubtract },
            { Symbols.FloatMultiply, OpCode.FloatMultiply },
            { Symbols.FloatDivide, OpCode.FloatDivide },
            { Symbols.FloatSmallerThan, OpCode.FloatSmallerThan },
            { Symbols.FloatGreaterThan, OpCode.FloatGreaterThan },
            { Symbols.FloatEquals, OpCode.FloatEquals },
            { Symbols.FloatMin, OpCode.FloatMin },
            { Symbols.FloatMax, OpCode.FloatMax },
            { Symbols.FloatCos, OpCode.FloatCos },
            { Symbols.FloatSin, OpCode.FloatSin },
            { Symbols.FloatTan, OpCode.FloatTan },
            { Symbols.FloatFromBoolean, OpCode.FloatFromBoolean },
            { Symbols.FloatFromInteger, OpCode.FloatFromInteger },
            { Symbols.FloatRand, OpCode.FloatRand },
            { Symbols.FloatRot, OpCode.FloatRot },
            { Symbols.FloatShove, OpCode.FloatShove },
            { Symbols.FloatStackDepth, OpCode.FloatStackDepth },
            { Symbols.FloatSwap, OpCode.FloatSwap },
            { Symbols.FloatYank, OpCode.FloatYank },
            { Symbols.FloatYankDuplicate, OpCode.FloatYankDuplicate },
            #endregion Float
        };

        public static OpCode GetOpCode(string operationName)
        {
            OpCode opCode;
            if (symbolTable.TryGetValue(operationName, out opCode)) return opCode;
            else throw new NotSupportedException("Operation not supported: " + operationName);
        }

        public static bool TryGetOpCode(string operationName, out OpCode opCode)
        {
            return symbolTable.TryGetValue(operationName, out opCode);
        }
    }
}
